#!/bin/bash -ex
# This script generates a chain of 100 modules, and a script to build them.
# Usage:
#   ./gen.sh && modules_repro/build.sh && ./print.sh

: "${CLANG:=clang}"
LIMIT=100
DIR="$PWD/modules_repro"
rm -r "$DIR" || true
mkdir -p "$DIR"
DLIUB="$DIR/dliub.hs"

# constant.modulemap is supplied to every compile.
# It contains only textual headers, and there is no PCM for it.
# It's large. (In real life, it contains full paths to clang builtin headers).
cat > "$DIR/constant.modulemap" <<-EOF
module "constant" {
  textual header "constant.h"
}
EOF
for i in `seq 1000`; do 
  echo "// make this really long" >> "$DIR/constant.modulemap"
done
# constant.h is a textual header from the constant module.
# It is rarely actually used - in this example nothing includes it.
cat > "$DIR/constant.h" <<-EOF
#pragma once
// hello, world!
EOF

# m1 to m100 are a chain of dependent modules, m1 -> m2 -> m3 ...
for i in `seq 1 $LIMIT`; do
  mkdir -p "$DIR/m$i"
  echo "module m$i { header \"header\" }" > "$DIR/m$i/map"
  next="$((i+1))"
  echo "#pragma once" > "$DIR/m$i/header"
  echo "#include \"m$next/header\"" >> "$DIR/m$i/header"
  echo "$CLANG -cc1 -xc++-header -fmodules -emit-module -o m$i/pcm -I. "\
       "m$i/map -fmodule-name=m$i -fmodule-map-file=m$i/map "\
       "-fmodule-map-file=constant.modulemap "\
       "-fmodule-map-file=m$next/map -fmodule-file=m$next=m$next/pcm"\
       >> "$DLIUB"
done
# Create a dummy m101 to make m100 work
mkdir -p "$DIR/m$((LIMIT+1))"
touch "$DIR/m$((LIMIT+1))/header"
touch "$DIR/m$((LIMIT+1))/map"

# Compile commands are in the wrong order, so fix that.
echo "#!/bin/bash -ex" > "$DIR/build.sh"
echo 'cd "$(dirname "$0")"' >> "$DIR/build.sh"
tac "$DIR/dliub.hs" >> "$DIR/build.sh"
chmod 755 "$DIR/build.sh"
